{% extends "basecontrol.html" %}
{% block control %}
<select name="action.{{ mod_alert.short_name }}.param.{{ param.name }}" id="{{ mod_alert.short_name }}_{{ param.name }}">
    {% for option, value in param.possible_values.items() %}
        <option value="{{ value }}">{{ option }}</option>
    {% endfor %}
</select>
{% endblock %}